@@ -28,9 +28,10 @@
 #include "iso14443a.h"
 #include "protocols.h"
 #include "cmd.h"
+#include "commonutil.h"
 
 void ModInfo(void) {
-    DbpString("  HF - Reading Visa cards & Emulating a Visa MSD Transaction(ISO14443) - (Salvador Mendoza)");
+    DbpString("  HF - Reading VISA cards & Emulating a VISA MSD Transaction(ISO14443) - (Salvador Mendoza)");
 }
 
 /* This standalone implements two different modes: reading and emulating.
@@ -132,9 +133,15 @@ static uint8_t treatPDOL(const uint8_t *apdu) {
 
 void RunMod(void) {
     StandAloneMode();
-    DbpString(_YELLOW_(">>") "Reading Visa cards & Emulating a Visa MSD Transaction a.k.a. MSDSal Started " _YELLOW_("<<"));
+    DbpString("");
+    DbpString(_YELLOW_(">>>") " Reading VISA cards & Emulating a VISA MSD Transaction a.k.a. MSDSal Started " _YELLOW_("<<<"));
+    DbpString("");
     FpgaDownloadAndGo(FPGA_BITSTREAM_HF);
 
+    // free eventually allocated BigBuf memory but keep Emulator Memory
+    // also sets HIGH pointer of BigBuf enabling us to malloc w/o fiddling w the reserved emulator memory
+    BigBuf_free_keep_EM();
+
     //For reading process
     iso14a_card_select_t card_a_info;
 
@@ -146,10 +153,32 @@ void RunMod(void) {
         0x59, 0x2e, 0x53, 0x59, 0x53, 0x2e, 0x44, 0x44,
         0x46, 0x30, 0x31, 0x00
     };
-    uint8_t visa[13] = {
-        0x00, 0xA4, 0x04, 0x00, 0x07, 0xa0, 0x00, 0x00,
-        0x00, 0x03, 0x10, 0x10, 0x00
+
+    uint8_t visa[] = { 0x00, 0xA4, 0x04, 0x00, 0x07, 0xa0, 0x00, 0x00, 0x00, 0x03, 0x10, 0x10, 0x00 };
+
+
+    /*
+    uint8_t select_aid_hdr[5] = { 0x00, 0xA4, 0x04, 0x00, 0x00 };
+    static const char* aid_list [] = {
+        "A00000000305076010",  // VISA ELO Credit
+        "A0000000031010",      // VISA Debit/Credit (Classic)
+        "A000000003101001",    // VISA Credit
+        "A000000003101002",    // VISA Debit
+        "A0000000032010",      // VISA Electron
+        "A0000000032020",      // VISA
+        "A0000000033010",      // VISA Interlink
+        "A0000000034010",      // VISA Specific
+        "A0000000035010",      // VISA Specific
+        "A0000000036010",      // Domestic Visa Cash Stored Value
+        "A0000000036020",      // International Visa Cash Stored Value
+        "A0000000038002",      // VISA Auth, VisaRemAuthen EMV-CAP (DPA)
+        "A0000000038010",      // VISA Plus
+        "A0000000039010",      // VISA Loyalty
+        "A000000003999910",    // VISA Proprietary ATM
+        "A000000098",          // Visa USA Debit Card
+        "A0000000980848",      // Visa USA Debit Cardv
     };
+    */
 
     uint8_t processing [8] = {0x80, 0xA8, 0x00, 0x00, 0x02, 0x83, 0x00, 0x00};
     uint8_t sfi[5] = {0x00, 0xb2, 0x01, 0x0c, 0x00};
@@ -161,48 +190,49 @@ void RunMod(void) {
 
     bool existpdol;
 
-
     // - MSD token card format -
     //
-    //Card number: 4412 3456 0578 1234
-    //Expiration date: 17/11
-    //Service code: 201
-    //Discretionary data: 0000030000991
-    //char token[19] = {0x44,0x12,0x34,0x56,0x05,0x78,0x12,0x34,0xd1,0x71,0x12,0x01,0x00,0x00,0x03,0x00,0x00,0x99,0x1f};
+    // Card number.............. 4412 3456 0578 1234
+    // Expiration date.......... 17/11
+    // Service code............. 201
+    // Discretionary data....... 0000030000991
+    // Pin verification value... 0000
+    // CVV / iCvv...............     030
+    // Trailing.................        000991
+
+    //                     44   12   34   56   05   78   12   34   D 1711     2   01   00   00   03   00   00   99   1
+    // char token[19] = {0x44,0x12,0x34,0x56,0x05,0x78,0x12,0x34,0xd1,0x71,0x12,0x01,0x00,0x00,0x03,0x00,0x00,0x99,0x1f};
     //
     // It is possible to initialize directly the emulation mode, having "token" with data and set "chktoken" = true ;)
     //
     char token[19] = {0x00};
     bool chktoken = false;
 
-    // Allocate 512 bytes for the dynamic modulation, created when the reader queries for it
-    // Such a response is less time critical, so we can prepare them on the fly
-#define DYNAMIC_RESPONSE_BUFFER_SIZE 64
-#define DYNAMIC_MODULATION_BUFFER_SIZE 512
-
     // UID 4 bytes(could be 7 bytes if needed it)
     uint8_t flags = FLAG_4B_UID_IN_DATA;
     // in case there is a read command received we shouldn't break
     uint8_t data[PM3_CMD_DATA_SIZE] = {0x00};
 
-    uint8_t visauid[7] = {0x01, 0x02, 0x03, 0x04};
+    uint8_t visauid[7] = {0xE9, 0x66, 0x5D, 0x20};
     memcpy(data, visauid, 4);
 
     // to initialize the emulation
-    uint8_t tagType = 11; // 11 = ISO/IEC 14443-4 - javacard (JCOP)
     tag_response_info_t *responses;
+
     uint32_t cuid = 0;
-    uint32_t counters[3] = { 0x00, 0x00, 0x00 };
-    uint8_t tearings[3] = { 0xbd, 0xbd, 0xbd };
-    uint8_t pages = 0;
 
     // command buffers
     uint8_t receivedCmd[MAX_FRAME_SIZE] = { 0x00 };
     uint8_t receivedCmdPar[MAX_PARITY_SIZE] = { 0x00 };
 
+
+    // Allocate 512 bytes for the dynamic modulation, created when the reader queries for it
+    // Such a response is less time critical, so we can prepare them on the fly
+#define DYNAMIC_RESPONSE_BUFFER_SIZE 64
+#define DYNAMIC_MODULATION_BUFFER_SIZE 512
+
     uint8_t dynamic_response_buffer[DYNAMIC_RESPONSE_BUFFER_SIZE] = {0};
     uint8_t dynamic_modulation_buffer[DYNAMIC_MODULATION_BUFFER_SIZE] = {0};
-
     // to know the transaction status
     uint8_t prevCmd = 0;
 
@@ -223,11 +253,11 @@ void RunMod(void) {
     // Checking if the user wants to go directly to emulation mode using a hardcoded track 2
     if (chktoken == true && token[0] != 0x00) {
         state = STATE_EMU;
-        DbpString(_YELLOW_("[ ") "Initialized emulation mode" _YELLOW_(" ]"));
-        DbpString("\n"_YELLOW_("!!") "Waiting for a card reader...");
+        DbpString("Initialized [ " _BLUE_("emulation mode") " ]");
+        DbpString("Waiting for a card reader...");
     } else {
-        DbpString(_YELLOW_("[ ") "Initialized reading mode" _YELLOW_(" ]"));
-        DbpString("\n"_YELLOW_("!!") "Waiting for a Visa card...");
+        DbpString("Initialized [ " _YELLOW_("reading mode") " ]");
+        DbpString("Waiting for a VISA card...");
     }
 
     for (;;) {
@@ -240,20 +270,20 @@ void RunMod(void) {
         int button_pressed = BUTTON_HELD(1000);
 
 
-        if (button_pressed  == BUTTON_HOLD)
+        if (button_pressed  == BUTTON_HOLD) {
             break;
-        else if (button_pressed == BUTTON_SINGLE_CLICK) {
+        } else if (button_pressed == BUTTON_SINGLE_CLICK) {
             // pressing one time change between reading & emulation
             if (state == STATE_READ) {
                 if (chktoken == true && token[0] != 0x00) {
                     // only change to emulation if it saved a track 2 in memory
                     state = STATE_EMU;
-                    DbpString(_YELLOW_("[ ") "In emulation mode" _YELLOW_(" ]"));
+                    DbpString("[ " _BLUE_("Emulation mode") " ]");
                 } else
-                    DbpString(_YELLOW_("!!") "Nothing in memory to emulate");
+                    DbpString(_YELLOW_("Nothing in memory to emulate"));
             } else {
                 state = STATE_READ;
-                DbpString(_YELLOW_("[ ") "In reading mode" _YELLOW_(" ]"));
+                DbpString("[ " _YELLOW_("Reading mode") " ]");
             }
         }
 
@@ -261,35 +291,40 @@ void RunMod(void) {
 
         if (state == STATE_READ) {
             LED_A_ON();
-            if (chktoken)
+            if (chktoken) {
                 LED_C_ON();
+            }
 
             iso14443a_setup(FPGA_HF_ISO14443A_READER_MOD);
 
             if (iso14443a_select_card(NULL, &card_a_info, NULL, true, 0, false)) {
 
-                DbpString(_YELLOW_("+") "Found ISO 14443 Type A!");
-
                 for (uint8_t i = 0; i < 4; i++) {
                     chktoken = false;
                     LED_C_OFF();
                     LED_B_ON();
+
+                    // add loop visa
+                    // for (int i = 0; i < ARRAYLEN(AIDlist); i ++) {
+//    hexstr_to_byte_array("a0da02631a440a44000000a012ad10a00e800200048108", sam_apdu, &sam_len);
                     uint8_t apdulen = iso14_apdu(apdus[i], (uint16_t) apduslen[i], false, apdubuffer, NULL);
 
                     if (apdulen > 0) {
-                        DbpString(_YELLOW_("[ ") "Proxmark command" _YELLOW_(" ]"));
+                        DbpString("[ " _YELLOW_("Proxmark command") " ]");
                         Dbhexdump(apduslen[i], apdus[i], false);
-                        DbpString(_GREEN_("[ ") "Card answer" _GREEN_(" ]"));
+                        DbpString("[ " _GREEN_("Card answer") " ]");
                         Dbhexdump(apdulen - 2, apdubuffer, false);
-                        DbpString("----");
+                        DbpString("-------------------------------");
 
                         for (uint8_t u = 0; u < apdulen; u++) {
                             if (i == 1) {
 
                                 // check for PDOL
                                 if (apdubuffer[u] == 0x9F && apdubuffer[u + 1] == 0x38) {
-                                    for (uint8_t e = 0; e <= apdubuffer[u + 2]; e++)
+
+                                    for (uint8_t e = 0; e <= apdubuffer[u + 2]; e++) {
                                         pdol[e] =  apdubuffer[u + e + 2];
+                                    }
 
                                     // generate a challenge
                                     plen = treatPDOL(pdol);
@@ -309,25 +344,27 @@ void RunMod(void) {
                         }
 
                         if (i == 1) {
-                            DbpString(_GREEN_("[ ") "Challenge generated" _GREEN_(" ]"));
+                            DbpString("[ "_GREEN_("Challenge generated") " ]");
                             Dbhexdump(plen, existpdol ? ppdol : processing, false);
                         }
                     } else {
-                        DbpString(_YELLOW_("!!") "Error reading the card");
+                        DbpString(_RED_("Error reading the card"));
                     }
                     LED_B_OFF();
                 }
 
                 if (chktoken) {
-                    DbpString(_RED_("[ ") "Track 2" _RED_(" ]"));
+                    DbpString("[ " _GREEN_("Track 2") " ]");
                     Dbhexdump(19, (uint8_t *)token, false);
-                    DbpString(_YELLOW_("!!") "Card number");
+                    DbpString("[ " _GREEN_("Card Number") " ]");
                     Dbhexdump(8, (uint8_t *)token, false);
-                    DbpString("---");
+                    DbpString("-------------------------------");
+                    DbpString("");
+                    DbpString("");
                     LED_C_ON();
                     state = STATE_EMU;
-                    DbpString(_YELLOW_("[ ") "Initialized emulation mode" _YELLOW_(" ]"));
-                    DbpString("\n"_YELLOW_("!!") "Waiting for a card reader...");
+                    DbpString("Initialized [ " _BLUE_("emulation mode") " ]");
+                    DbpString("Waiting for a card reader...");
                 }
             }
             FpgaWriteConfWord(FPGA_MAJOR_MODE_OFF);
@@ -340,14 +377,15 @@ void RunMod(void) {
             // free eventually allocated BigBuf memory but keep Emulator Memory
             BigBuf_free_keep_EM();
 
-            if (SimulateIso14443aInit(tagType, flags, data, &responses, &cuid, counters, tearings, &pages) == false) {
+            // tag type: 11 = ISO/IEC 14443-4 - javacard (JCOP)
+            if (SimulateIso14443aInit(11, flags, data, &responses, &cuid, NULL, NULL, NULL) == false) {
                 BigBuf_free_keep_EM();
                 reply_ng(CMD_HF_MIFARE_SIMULATE, PM3_EINIT, NULL, 0);
-                DbpString(_YELLOW_("!!") "Error initializing the emulation process!");
+                DbpString(_RED_("Error initializing the emulation process!"));
                 SpinDelay(500);
                 state = STATE_READ;
-                DbpString(_YELLOW_("[ ") "Initialized reading mode" _YELLOW_(" ]"));
-                DbpString("\n" _YELLOW_("!!") "Waiting for a Visa card...");
+                DbpString("Initialized [ "_YELLOW_("reading mode") " ]");
+                DbpString("Waiting for a VISA card...");
                 continue;
             }
 
@@ -366,11 +404,12 @@ void RunMod(void) {
             for (;;) {
                 LED_B_OFF();
                 // clean receive command buffer
-                if (!GetIso14443aCommandFromReader(receivedCmd, receivedCmdPar, &len)) {
-                    DbpString(_YELLOW_("!!") "Emulator stopped");
+                if (GetIso14443aCommandFromReader(receivedCmd, receivedCmdPar, &len) == false) {
+                    DbpString("Emulator stopped");
                     retval = PM3_EOPABORTED;
                     break;
                 }
+
                 tag_response_info_t *p_response = NULL;
                 LED_B_ON();
 
@@ -387,53 +426,49 @@ void RunMod(void) {
 
                     // received a HALT
                 } else if (receivedCmd[0] == ISO14443A_CMD_HALT && len == 4) {
-                    //DbpString(_YELLOW_("+") "Received a HALT");
                     p_response = NULL;
 
                     // received a WAKEUP
                 } else if (receivedCmd[0] == ISO14443A_CMD_WUPA && len == 1) {
-                    //DbpString(_YELLOW_("+") "WAKEUP Received");
                     prevCmd = 0;
                     p_response = &responses[RESP_INDEX_ATQA];
 
                     // received request for UID (cascade 1)
                 } else if (receivedCmd[1] == 0x20 && receivedCmd[0] == ISO14443A_CMD_ANTICOLL_OR_SELECT && len == 2) {
-                    //DbpString(_YELLOW_("+") "Request for UID C1");
                     p_response = &responses[RESP_INDEX_UIDC1];
 
                     // received a SELECT (cascade 1)
                 } else if (receivedCmd[1] == 0x70 && receivedCmd[0] == ISO14443A_CMD_ANTICOLL_OR_SELECT && len == 9) {
-                    //DbpString(_YELLOW_("+") "Request for SELECT S1");
                     p_response = &responses[RESP_INDEX_SAKC1];
 
                     // received a RATS request
                 } else if (receivedCmd[0] == ISO14443A_CMD_RATS && len == 4) {
-                    DbpString(_YELLOW_("+") "Request for RATS");
                     prevCmd = 0;
-                    //p_response = &responses[RESP_INDEX_RATS];
-
-                    static uint8_t rRATS[] = { 0x13, 0x78, 0x80, 0x72, 0x02, 0x80, 0x31, 0x80, 0x66, 0xb1, 0x84, 0x0c, 0x01, 0x6e, 0x01, 0x83, 0x00, 0x90, 0x00 };
-
-                    memcpy(&dynamic_response_info.response[0], rRATS, sizeof(rRATS));
-                    dynamic_response_info.response_n = sizeof(rRATS);
+                    p_response = &responses[RESP_INDEX_RATS];
 
                 } else {
-                    DbpString(_YELLOW_("[ ") "Card reader command" _YELLOW_(" ]"));
-                    Dbhexdump(len, receivedCmd, false);
+                    if (g_dbglevel == DBG_DEBUG) {
+                        DbpString("[ "_YELLOW_("Card reader command") " ]");
+                        Dbhexdump(len, receivedCmd, false);
+                    }
 
-                    // emulate a Visa MSD(Magnetic stripe data) card
+                    // emulate a Visa MSD (Magnetic stripe data) card
                     if (receivedCmd[0] == 0x02 || receivedCmd[0] == 0x03) {
                         dynamic_response_info.response[0] = receivedCmd[0];
 
                         // depending on card reader commands, the Proxmark will answer to fool the reader
                         // respond with PPSE
                         if (receivedCmd[2] == 0xA4 && receivedCmd[6] == 0x32 && prevCmd == 0) {
+                            // need to adapt lengths..
                             uint8_t ppsea[39] = {
+                                //        0x23 = 35,  skip two first bytes then the message - SW 2 is 35 = 0x23
                                 0x6F, 0x23, 0x84, 0x0E, 0x32, 0x50, 0x41, 0x59,
                                 0x2E, 0x53, 0x59, 0x53, 0x2E, 0x44, 0x44, 0x46,
                                 0x30, 0x31, 0xA5, 0x11, 0xBF, 0x0C, 0x0E, 0x61,
-                                0x0C, 0x4F, 0x07, 0xA0, 0x00, 0x00, 0x00, 0x03,
-                                0x10, 0x10, 0x87, 0x01, 0x01, 0x90, 0x00
+                                0x0C, 0x4F,
+                                //  len   aid0  aid1  aid2...
+                                0x07, 0xA0, 0x00, 0x00, 0x00, 0x03, 0x10, 0x10,
+                                0x87, 0x01, 0x01, 0x90, 0x00
                             };
                             memcpy(&dynamic_response_info.response[1], ppsea, sizeof(ppsea));
                             dynamic_response_info.response_n = sizeof(ppsea) + 1;
@@ -442,10 +477,14 @@ void RunMod(void) {
                             // respond Visa AID
                         } else if (receivedCmd[2] == 0xA4 && receivedCmd[10] == 0x03 && receivedCmd[11] == 0x10 && prevCmd == 1) {
                             uint8_t visauid_long[34] = {
-                                0x6F, 0x1E, 0x84, 0x07, 0xA0, 0x00, 0x00, 0x00,
-                                0x03, 0x10, 0x10, 0xA5, 0x13, 0x50, 0x0B, 0x56,
-                                0x49, 0x53, 0x41, 0x20, 0x43, 0x52, 0x45, 0x44,
-                                0x49, 0x54, 0x9F, 0x38, 0x03, 0x9F, 0x66, 0x02,
+                                //          0x1E = 30,  skip two first bytes then the message - SW 2 is 30 = 0x1E
+                                0x6F, 0x1E, 0x84,
+                                //      len   aid0  aid1  aid2....
+                                0x07, 0xA0, 0x00, 0x00, 0x00, 0x03, 0x10, 0x10,
+                                0xA5, 0x13, 0x50,
+                                //      len      V     I     S     A           C     R    E      D     I     T
+                                0x0B, 0x56, 0x49, 0x53, 0x41, 0x20, 0x43, 0x52, 0x45, 0x44, 0x49, 0x54,
+                                0x9F, 0x38, 0x03, 0x9F, 0x66, 0x02,
                                 0x90, 0x00
                             };
                             memcpy(&dynamic_response_info.response[1], visauid_long, sizeof(visauid_long));
@@ -461,18 +500,21 @@ void RunMod(void) {
 
                             // SFI
                         } else if (receivedCmd[1] == 0x00 && receivedCmd[2] == 0xB2  && prevCmd == 3) {
-                            uint8_t last[4] =  {0x70, 0x15, 0x57, 0x13};
-                            uint8_t statusapdu[2] = {0x90, 0x00};
-                            uint8_t card[25];
-                            memcpy(&card[0], last, sizeof(last));
+                            uint8_t card[25] = {
+                                0x70, 0x15, 0x57, 0x13, 0x00, 0x00, 0x00, 0x00,
+                                0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+                                0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+                                0x90, 0x00
+                            };
+                            // add token array == Track 2 found before
                             memcpy(&card[4], token, sizeof(token));
-                            memcpy(&card[23], statusapdu, sizeof(statusapdu));
+
                             memcpy(&dynamic_response_info.response[1], card, sizeof(card));
                             dynamic_response_info.response_n = sizeof(card) + 1;
                             prevCmd++;
 
                         } else {
-                            uint8_t finished[2] = {0x6f, 0x00};
+                            uint8_t finished[2] = {0x6F, 0x00};
                             memcpy(&dynamic_response_info.response[1], finished, sizeof(finished));
                             dynamic_response_info.response_n = sizeof(finished) + 1;
                             if (prevCmd == 5) {
@@ -480,7 +522,7 @@ void RunMod(void) {
                             }
                         }
                     } else {
-                        DbpString(_YELLOW_("!!") "Received unknown command!");
+                        DbpString(_RED_("Received unknown command!"));
                         if (prevCmd < 4) {
                             memcpy(dynamic_response_info.response, receivedCmd, len);
                             dynamic_response_info.response_n = len;
@@ -491,7 +533,7 @@ void RunMod(void) {
                 }
 
                 if (dynamic_response_info.response_n > 0) {
-                    DbpString(_GREEN_("[ ") "Proxmark3 answer" _GREEN_(" ]"));
+                    DbpString("[ " _GREEN_("Proxmark3 answer") " ]");
                     Dbhexdump(dynamic_response_info.response_n, dynamic_response_info.response, false);
                     DbpString("----");
 
@@ -501,7 +543,7 @@ void RunMod(void) {
 
                     if (prepare_tag_modulation(&dynamic_response_info, DYNAMIC_MODULATION_BUFFER_SIZE) == false) {
                         SpinDelay(500);
-                        DbpString(_YELLOW_("!!") "Error preparing Proxmark to answer!");
+                        DbpString(_RED_("Error preparing Proxmark to answer!"));
                         continue;
                     }
                     p_response = &dynamic_response_info;
@@ -511,13 +553,15 @@ void RunMod(void) {
                     EmSendPrecompiledCmd(p_response);
                 }
             }
-            switch_off();
 
+            switch_off();
             set_tracing(false);
             BigBuf_free_keep_EM();
             reply_ng(CMD_HF_MIFARE_SIMULATE, retval, NULL, 0);
         }
     }
-    DbpString(_YELLOW_("[=]") "exiting");
+    DbpString("Exit standalone mode!");
+    DbpString("");
+    SpinErr(15, 200, 3);
     LEDsoff();
 }
