@@ -38,8 +38,6 @@ void iso14443b_setup(void);
 int iso14443b_apdu(uint8_t const *msg, size_t msg_len, bool send_chaining, void *rxdata, uint16_t rxmaxlen, uint8_t *res);
 
 int iso14443b_select_card(iso14b_card_select_t *card);
-int iso14443b_select_card_srx(iso14b_card_select_t *card);
-int iso14443b_select_xrx_card(iso14b_card_select_t *card);
 
 void SimulateIso14443bTag(const uint8_t *pupi);
 void AcquireRawAdcSamplesIso14443b(uint32_t parameter);
