@@ -268,8 +268,9 @@ int BUTTON_HELD(int ms) {
     int ticks = (48000 * (ms ? ms : 1000)) >> 10;
 
     // If we're not even pressed, forget about it!
-    if (BUTTON_PRESS() == false)
+    if (BUTTON_PRESS() == false) {
         return BUTTON_NO_CLICK;
+    }
 
     // Borrow a PWM unit for my real-time clock
     AT91C_BASE_PWMC->PWMC_ENA = PWM_CHANNEL(0);
@@ -284,12 +285,14 @@ int BUTTON_HELD(int ms) {
         uint16_t now = AT91C_BASE_PWMC_CH0->PWMC_CCNTR;
 
         // As soon as our button let go, we didn't hold long enough
-        if (BUTTON_PRESS() == false)
+        if (BUTTON_PRESS() == false) {
             return BUTTON_SINGLE_CLICK;
+        }
 
         // Have we waited the full second?
-        else if (now == (uint16_t)(start + ticks))
+        else if (now == (uint16_t)(start + ticks)) {
             return BUTTON_HOLD;
+        }
 
         WDT_HIT();
     }
@@ -298,6 +301,8 @@ int BUTTON_HELD(int ms) {
     return BUTTON_ERROR;
 }
 
+// This function returns false if no data is available or
+// the USB connection is invalid.
 bool data_available(void) {
 #ifdef WITH_FPC_USART_HOST
     return usb_poll_validate_length() || (usart_rxdata_available() > 0);
@@ -305,3 +310,14 @@ bool data_available(void) {
     return usb_poll_validate_length();
 #endif
 }
+
+// This function doesn't check if the USB connection is valid.
+// In most of the cases, you should use data_available() unless
+// the timing is critical.
+bool data_available_fast(void) {
+#ifdef WITH_FPC_USART_HOST
+    return usb_available_length() || (usart_rxdata_available() > 0);
+#else
+    return usb_available_length();
+#endif
+}
