@@ -31,11 +31,13 @@
 #include "cmdhw.h"
 #include "cmddata.h"
 #include "commonutil.h"
+#include "preferences.h"
 #include "pm3_cmd.h"
 #include "pmflash.h"      // rdv40validation_t
 #include "cmdflashmem.h"  // get_signature..
 #include "uart/uart.h" // configure timeout
 #include "util_posix.h"
+#include "flash.h" // reboot to bootloader mode
 
 static int CmdHelp(const char *Cmd);
 
@@ -476,14 +478,9 @@ static int CmdDbg(const char *Cmd) {
         return PM3_EINVARG;
     }
 
-    clearCommandBuffer();
-    SendCommandNG(CMD_GET_DBGMODE, NULL, 0);
-    PacketResponseNG resp;
-    if (WaitForResponseTimeout(CMD_GET_DBGMODE, &resp, 2000) == false) {
-        PrintAndLogEx(WARNING, "Failed to get current device debug level");
-        return PM3_ETIMEOUT;
-    }
-    uint8_t curr = resp.data.asBytes[0];
+    uint8_t curr = DBG_NONE;
+    if (getDeviceDebugLevel(&curr) != PM3_SUCCESS)
+        return PM3_EFAILED;
 
     const char *dbglvlstr;
     switch (curr) {
@@ -522,8 +519,8 @@ static int CmdDbg(const char *Cmd) {
         else if (lv4)
             dbg = 4;
 
-        clearCommandBuffer();
-        SendCommandNG(CMD_SET_DBGMODE, &dbg, sizeof(dbg));
+        if (setDeviceDebugLevel(dbg, true) != PM3_SUCCESS)
+            return PM3_EFAILED;
     }
     return PM3_SUCCESS;
 }
@@ -1104,8 +1101,27 @@ static int CmdBreak(const char *Cmd) {
     return PM3_SUCCESS;
 }
 
+static int CmdBootloader(const char *Cmd) {
+
+    CLIParserContext *ctx;
+    CLIParserInit(&ctx, "hw bootloader",
+                  "Reboot Proxmark3 into bootloader mode",
+                  "hw bootloader\n"
+                 );
+
+    void *argtable[] = {
+        arg_param_begin,
+        arg_param_end
+    };
+    CLIExecWithReturn(ctx, Cmd, argtable, true);
+    CLIParserFree(ctx);
+    clearCommandBuffer();
+    flash_reboot_bootloader(g_conn.serial_port_name, false);
+    return PM3_SUCCESS;
+}
+
 int set_fpga_mode(uint8_t mode) {
-    if (mode < 1 || mode > 4) {
+    if (mode < FPGA_BITSTREAM_LF || mode > FPGA_BITSTREAM_HF_15) {
         return PM3_EINVARG;
     }
     uint8_t d[] = {mode};
@@ -1126,6 +1142,7 @@ static command_t CommandTable[] = {
     {"-------------", CmdHelp,         AlwaysAvailable, "----------------------- " _CYAN_("Hardware") " -----------------------"},
     {"help",          CmdHelp,         AlwaysAvailable, "This help"},
     {"break",         CmdBreak,        IfPm3Present,    "Send break loop usb command"},
+    {"bootloader",    CmdBootloader,   IfPm3Present,    "Reboot Proxmark3 into bootloader mode"},
     {"connect",       CmdConnect,      AlwaysAvailable, "Connect Proxmark3 to serial port"},
     {"dbg",           CmdDbg,          IfPm3Present,    "Set Proxmark3 debug level"},
     {"detectreader",  CmdDetectReader, IfPm3Present,    "Detect external reader field"},
