@@ -91,6 +91,23 @@ static uint8_t calc_pos(const uint8_t *d) {
     return pos;
 }
 
+// Copy an existing buffer into client trace buffer
+// I think this is cleaner than further globalizing gs_trace, and may lend itself to more modularity later?
+bool ImportTraceBuffer(uint8_t *trace_src, uint16_t trace_len) {
+    if (trace_len == 0 || trace_src == NULL) return (false);
+    if (gs_trace) {
+        free(gs_trace);
+        gs_traceLen = 0;
+    }
+    gs_trace = calloc(trace_len, sizeof(uint8_t));
+    if (gs_trace == NULL) {
+        return (false);
+    }
+    memcpy(gs_trace, trace_src, trace_len);
+    gs_traceLen = trace_len;
+    return (true);
+}
+
 static uint8_t extract_uid[10] = {0};
 static uint8_t extract_uidlen = 0;
 static uint8_t extract_epurse[8] = {0};
@@ -912,22 +929,32 @@ static uint16_t printTraceLine(uint16_t tracepos, uint16_t traceLen, uint8_t *tr
         size_t mfDataLen = 0;
         if (DecodeMifareData(frame, data_len, parityBytes, hdr->isResponse, mfData, &mfDataLen, mfDicKeys, mfDicKeysCount)) {
             memset(explanation, 0x00, sizeof(explanation));
-            annotateIso14443a(explanation, sizeof(explanation), mfData, mfDataLen, hdr->isResponse);
+
+            if (protocol == PROTO_MFPLUS) {
+                annotateMfPlus(explanation, sizeof(explanation), mfData, mfDataLen);
+            } else {
+                annotateIso14443a(explanation, sizeof(explanation), mfData, mfDataLen, hdr->isResponse);
+            }
             uint8_t crcc = iso14443A_CRC_check(hdr->isResponse, mfData, mfDataLen);
 
-            //iceman: colorise crc bytes here will need a refactor of code from above.
-            if (hdr->isResponse) {
-                PrintAndLogEx(NORMAL, "            |            |  *  |%-*s | %-4s| %s",
-                              str_padder,
-                              sprint_hex_inrow_spaces(mfData, mfDataLen, 2),
-                              (crcc == 0 ? _RED_(" !! ") : (crcc == 1 ? _GREEN_(" ok ") : "    ")),
-                              explanation);
-            } else {
-                PrintAndLogEx(NORMAL, "            |            |  *  |" _YELLOW_("%-*s")" | " _YELLOW_("%s") "| " _YELLOW_("%s"),
-                              str_padder,
-                              sprint_hex_inrow_spaces(mfData, mfDataLen, 2),
-                              (crcc == 0 ? _RED_(" !! ") : (crcc == 1 ? _GREEN_(" ok ") : "    ")),
-                              explanation);
+            // iceman: colorise crc bytes here will need a refactor of code from above.
+            for (int j = 0; j < mfDataLen; j += TRACE_MAX_HEX_BYTES) {
+
+                int plen = MIN((mfDataLen - j), TRACE_MAX_HEX_BYTES);
+
+                if (hdr->isResponse) {
+                    PrintAndLogEx(NORMAL, "            |            |  *  |%-*s | %-4s| %s",
+                                  str_padder,
+                                  sprint_hex_inrow_spaces(mfData + j, plen, 2),
+                                  (crcc == 0 ? _RED_(" !! ") : (crcc == 1 ? _GREEN_(" ok ") : "    ")),
+                                  explanation);
+                } else {
+                    PrintAndLogEx(NORMAL, "            |            |  *  |" _YELLOW_("%-*s")" | " _YELLOW_("%s") "| " _YELLOW_("%s"),
+                                  str_padder,
+                                  sprint_hex_inrow_spaces(mfData + j, plen, 2),
+                                  (crcc == 0 ? _RED_(" !! ") : (crcc == 1 ? _GREEN_(" ok ") : "    ")),
+                                  explanation);
+                }
             }
         }
     }
@@ -1094,8 +1121,9 @@ static int CmdTraceLoad(const char *Cmd) {
     CLIParserFree(ctx);
 
     if (gs_trace) {
-        free(gs_trace);
+        free(gs_trace); // maybe better to not clobber this until we have successful load?
         gs_trace = NULL;
+        gs_traceLen = 0;
     }
 
     size_t len = 0;
@@ -1282,7 +1310,7 @@ int CmdTraceList(const char *Cmd) {
 
     if (use_buffer == false) {
         download_trace();
-    } else if (gs_traceLen == 0) {
+    } else if (gs_traceLen == 0 || gs_trace == NULL) {
         PrintAndLogEx(FAILED, "You requested a trace list in offline mode but there is no trace.");
         PrintAndLogEx(FAILED, "Consider using " _YELLOW_("`trace load`") " or removing parameter " _YELLOW_("`-1`"));
         return PM3_EINVARG;
