@@ -63,7 +63,7 @@ void push6bits(BitstreamOut_t *stream, uint8_t bits);
 void EncryptDES(bool key[56], bool outBlk[64], bool inBlk[64], int verbose) ;
 void x_num_to_bytes(uint64_t n, size_t len, uint8_t *dest);
 uint64_t x_bytes_to_num(uint8_t *src, size_t len);
-uint8_t reversebytes(uint8_t b);
+uint8_t reversebyte(uint8_t b);
 void reverse_arraybytes(uint8_t *arr, size_t len);
 void reverse_arraycopy(uint8_t *arr, uint8_t *dest, size_t len);
 void printarr(const char *name, uint8_t *arr, int len);
