@@ -788,11 +788,11 @@ bool desfire_crc32_check(uint8_t *data, const size_t len, uint8_t *crc) {
 }
 
 void iso14443a_crc_append(uint8_t *data, size_t len) {
-    return compute_crc(CRC_14443_A, data, len, data + len, data + len + 1);
+    compute_crc(CRC_14443_A, data, len, data + len, data + len + 1);
 }
 
 void iso14443a_crc(uint8_t *data, size_t len, uint8_t *pbtCrc) {
-    return compute_crc(CRC_14443_A, data, len, pbtCrc, pbtCrc + 1);
+    compute_crc(CRC_14443_A, data, len, pbtCrc, pbtCrc + 1);
 }
 
 bool iso14443a_crc_check(uint8_t *data, const size_t len, uint8_t *crc) {
