@@ -322,6 +322,7 @@ const static vocabulary_t vocabulary[] = {
     { 0, "hf lto wrbl" },
     { 1, "hf mf help" },
     { 1, "hf mf list" },
+    { 0, "hf mf info" },
     { 0, "hf mf darkside" },
     { 0, "hf mf nested" },
     { 1, "hf mf hardnested" },
@@ -390,6 +391,8 @@ const static vocabulary_t vocabulary[] = {
     { 0, "hf mfp rdbl" },
     { 0, "hf mfp rdsc" },
     { 0, "hf mfp wrbl" },
+    { 0, "hf mfp chkey" },
+    { 0, "hf mfp chconf" },
     { 0, "hf mfp commitp" },
     { 0, "hf mfp initp" },
     { 0, "hf mfp wrp" },
@@ -506,6 +509,7 @@ const static vocabulary_t vocabulary[] = {
     { 0, "hf xerox dump" },
     { 1, "hw help" },
     { 0, "hw break" },
+    { 0, "hw bootloader" },
     { 1, "hw connect" },
     { 0, "hw dbg" },
     { 0, "hw detectreader" },
