@@ -20,12 +20,48 @@
 
 #import <Foundation/NSString.h>
 #import <Foundation/NSProcessInfo.h>
+
+#if TARGET_OS_IOS
+#import <UIKit/UIKit.h>
+#else
 #import <AppKit/NSApplication.h>
+#endif
+
+#if !defined(USING_ARC)
+#  if __has_feature(objc_arc)
+#     define USING_ARC 1
+#  else
+#    define USING_ARC 0
+#  endif
+#elif EMPTY_DEFINE(USING_ARC)
+#   undef USING_ARC
+#   define USING_ARC 1
+#endif
 
 static id activity = nil;
 
 //OS X Version 10.10 is defined in OS X 10.10 and later
 #if defined(MAC_OS_X_VERSION_10_10)
+#if USING_ARC
+@implementation AppDelegate {
+    id <NSObject> activity;
+}
+void disableAppNap(const char* reason) {
+    if(activity == nil) {
+        //NSLog(@"disableAppNap: %@", @(reason));
+        activity = [[NSProcessInfo processInfo] beginActivityWithOptions:NSActivityBackground reason:@(reason)];
+    }
+}
+
+void enableAppNap() {
+    if(activity != nil) {
+        //NSLog(@"enableAppNap");
+        [[NSProcessInfo processInfo] endActivity:activity];
+        activity = nil;
+    }
+}
+@end
+#else
 void disableAppNap(const char* reason) {
     if(activity == nil) {
         //NSLog(@"disableAppNap: %@", @(reason));
@@ -42,6 +78,7 @@ void enableAppNap() {
         activity = nil;
     }
 }
+#endif
 
 #else
 void disableAppNap(const char* reason) { }
@@ -49,8 +86,11 @@ void enableAppNap() { }
 #endif
 
 
+#if TARGET_OS_IOS
+void makeUnfocusable() { }
+void makeFocusable() { }
 //OS X Version 10.6 is defined in OS X 10.6 and later
-#if defined(MAC_OS_X_VERSION_10_6)
+#elif defined(MAC_OS_X_VERSION_10_6)
 void makeUnfocusable() {
     [NSApp setActivationPolicy:NSApplicationActivationPolicyProhibited];
 }
@@ -60,4 +100,4 @@ void makeFocusable() {
 #else
 void makeUnfocusable() { }
 void makeFocusable() { }
-#endif
\ No newline at end of file
+#endif
